
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "src/benchmark_func.h"

typedef long double (*bmfunc)(int, double *);

struct bmtool
{
   bmfunc F01;
   bmfunc F02;
   bmfunc F03;
   bmfunc F04;
   bmfunc F05;
   bmfunc F06;
   bmfunc F07;
   bmfunc F08;
   bmfunc F09;
   bmfunc F10;
   bmfunc F11;
   bmfunc F12;
   bmfunc F13;
   bmfunc F14;
   bmfunc F15;
   bmfunc F16;
   bmfunc F17;
   bmfunc F18;
   bmfunc F19;
};

static const char *bmfuncnames[] = 
{
   "",
   "Shifted_Sphere",
   "Schwefel_Problem",
   "Shifted_Rosenbrock",
   "Shifted_Rastrigin",
   "Shifted_Griewank",
   "Shifted_Ackley",
   "f_Schwefel2_22",
   "f_Schwefel1_2",
   "Extended_f_10",
   "f_Bohachevsky",
   "f_Schaffer",
   "f_Hybrid_12",
   "f_Hybrid_13",
   "f_Hybrid_14",
   "f_Hybrid_15",
   "f_Hybrid_16new",
   "f_Hybrid_17new",
   "f_Hybrid_18new",
   "f_Hybrid_19new"
};

static int LoadFlag = 0;
static bmfunc bmfuncs[20];
static struct bmtool *bmtool;
static struct bmtool bmtoolstruct;


static void bmfuncload(void)
{
   void *Handle;
   char bmSharedLibName[160];

   bmtool = &bmtoolstruct;

   sprintf(bmSharedLibName, "lib/f1tof6func.so");
   Handle = dlopen(bmSharedLibName, RTLD_LAZY);
   if (!Handle)
   {
      fprintf(stderr, "Fail to load lib/f1tof6func.so: %s\n", dlerror());
      return;
   }
   
   bmtool->F01 = dlsym(Handle, bmfuncnames[1]);
   bmtool->F02 = dlsym(Handle, bmfuncnames[2]);
   bmtool->F03 = dlsym(Handle, bmfuncnames[3]);
   bmtool->F04 = dlsym(Handle, bmfuncnames[4]);
   bmtool->F05 = dlsym(Handle, bmfuncnames[5]);
   bmtool->F06 = dlsym(Handle, bmfuncnames[6]);

   sprintf(bmSharedLibName, "lib/f7tof11func.so");
   Handle = dlopen(bmSharedLibName, RTLD_LAZY);
   if (!Handle)
   {
      fprintf(stderr, "Fail to load lib/f7tof11func.so: %s\n", dlerror());
      return;
   }
   
   bmtool->F07 = dlsym(Handle, bmfuncnames[7]);
   bmtool->F08 = dlsym(Handle, bmfuncnames[8]);
   bmtool->F09 = dlsym(Handle, bmfuncnames[9]);
   bmtool->F10 = dlsym(Handle, bmfuncnames[10]);
   bmtool->F11 = dlsym(Handle, bmfuncnames[11]);
   
   sprintf(bmSharedLibName, "lib/f12tof19func.so");
   Handle = dlopen(bmSharedLibName, RTLD_LAZY);
   if (!Handle)
   {
      fprintf(stderr, "Fail to load lib/f12tof19func.so: %s\n", dlerror());
      return;
   }
   
   bmtool->F12 = dlsym(Handle, bmfuncnames[12]);
   bmtool->F13 = dlsym(Handle, bmfuncnames[13]);
   bmtool->F14 = dlsym(Handle, bmfuncnames[14]);
   bmtool->F15 = dlsym(Handle, bmfuncnames[15]);
   bmtool->F16 = dlsym(Handle, bmfuncnames[16]);
   bmtool->F17 = dlsym(Handle, bmfuncnames[17]);
   bmtool->F18 = dlsym(Handle, bmfuncnames[18]);
   bmtool->F19 = dlsym(Handle, bmfuncnames[19]);
}


static void bmfuncmapping(void)
{
   int func_num;

   for (func_num = 1; func_num <= 19; func_num++)
   {
      switch (func_num)
      {
         case 1:
            bmfuncs[func_num] = bmtool->F01;
            break;
         case 2:
            bmfuncs[func_num] = bmtool->F02;
            break;
         case 3:
            bmfuncs[func_num] = bmtool->F03;
            break;
         case 4:
            bmfuncs[func_num] = bmtool->F04;
            break;
         case 5:
            bmfuncs[func_num] = bmtool->F05;
            break;
         case 6:
            bmfuncs[func_num] = bmtool->F06;
            break;
         case 7:
            bmfuncs[func_num] = bmtool->F07;
            break;
         case 8:
            bmfuncs[func_num] = bmtool->F08;
            break;
         case 9:
            bmfuncs[func_num] = bmtool->F09;
            break;
         case 10:
            bmfuncs[func_num] = bmtool->F10;
            break;
         case 11:
            bmfuncs[func_num] = bmtool->F11;
            break;
         case 12:
            bmfuncs[func_num] = bmtool->F12;
            break;
         case 13:
            bmfuncs[func_num] = bmtool->F13;
            break;
         case 14:
            bmfuncs[func_num] = bmtool->F14;
            break;
         case 15:
            bmfuncs[func_num] = bmtool->F15;
            break;
         case 16:
            bmfuncs[func_num] = bmtool->F16;
            break;
         case 17:
            bmfuncs[func_num] = bmtool->F17;
            break;
         case 18:
            bmfuncs[func_num] = bmtool->F18;
            break;
         case 19:
            bmfuncs[func_num] = bmtool->F19;
            break;
      }
   }
}


void benchmark_func(int func_num, int np, Individual *pop)
{
   int i;

   if (!LoadFlag)
   {
      LoadFlag = 1;
      bmfuncload();
      bmfuncmapping();
   }

   assert(func_num >= 1);
   assert(func_num <= MAXFUNCID);
   assert(pop[0].dim >= 1);
   assert(pop[0].dim <= MAXD);

   for (i = 0; i < np; i++)
   {
      pop[i].val = (bmfuncs[func_num])(pop[i].dim, pop[i].x);
   }
}

