
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "src/cmdln_gade.h"

static int MaxFEs;
static int Dimension;
static int Population;
static double SAfactor;
static int StartRun;
static int EndRun;
static int StartFunction;
static int EndFunction;
static int RandSeed;
static int Interval;

static const char *HelpText =
   "Usage: gade [OPTION] ...\n\n"\
   "where:\n"
   "   -h          display this help and exit\n"\
   "   -d int      specify the problem dimension, default d = 50\n"\
   "   -n int      specify the population size, default n = 60\n"\
   "   -c double   specify the self-adaptation factor, default c = 0.1\n"\
   "   -t int      specify the maximal FEs, default t = 5000*d\n"\
   "   -r run      specify the first run id\n"\
   "   -R run      specify the last run id\n"\
   "   -f func     specify the first function id\n"\
   "   -F func     specify the last function id\n"\
   "   -s seed     specify the random seed\n"\
   "   -i interval specify the output interval\n"\
   "\n";


int cmdLnRandSeed(void)
{
   return (RandSeed);
}

int cmdLnStartRun(void)
{
   return (StartRun);
}

int cmdLnEndRun(void)
{
   return (EndRun);
}

int cmdLnStartFunction(void)
{
   return (StartFunction);
}

int cmdLnEndFunction(void)
{
   return (EndFunction);
}

int cmdLnMaxFEs(void)
{
   return (MaxFEs);
}

int cmdLnDimension(void)
{
   return (Dimension);
}

int cmdLnPopulation(void)
{
   return (Population);
}

int cmdLnInterval(void)
{
   return (Interval);
}

double cmdLnSAfactor(void)
{
   return (SAfactor);
}

void cmdLnInit(int Argc, char *Argv[])
{
   int c;
   extern char *optarg;
   extern int optind, opterr, optopt;
   int FEsFlag = 0;
   int HelpFlag = 0;
   int RunFlag = 0;
   int FuncFlag = 0;
   int SeedFlag = 0;
   int IntervalFlag = 0;

   Dimension = 50;
   Population = 60;
   SAfactor = 0.1;
   StartRun = 1;
   StartFunction = 1;

   while ((c = getopt(Argc, Argv, "d:n:c:t:r:R:f:F:s:i:h")) != -1)
   {
      switch (c)
      {
         case 'h':
            HelpFlag = 1;
            break;
         case 'd':
            Dimension = atoi(optarg);
            break;
         case 'n':
            Population = atoi(optarg);
            break;
         case 'c':
            SAfactor = strtod(optarg, NULL);
            break;
         case 't':
            FEsFlag = 1;
            MaxFEs = atoi(optarg);
            break;
         case 'r':
            StartRun = atoi(optarg);
            break;
         case 'R':
            RunFlag = 1;
            EndRun = atoi(optarg);
            break;
         case 'f':
            StartFunction = atoi(optarg);
            break;
         case 'F':
            FuncFlag = 1;
            EndFunction = atoi(optarg);
            break;
         case 's':
            SeedFlag = 1;
            RandSeed = atoi(optarg);
            break;
         case 'i':
            IntervalFlag = 1;
            Interval = atoi(optarg);
            break;
         case '?':
            fprintf(stderr, "Unknown option %c was specified for gade\n", c);
            exit(-1);
            break;
         default:
            fprintf(stderr, "Unhandled option %c was specified for gade\n", c);
            exit(-2);
            break;
      }
   }

   if (HelpFlag)
   {
      fputs(HelpText, stdout);
      exit(0);
   }
   if (!FEsFlag)
   {
      MaxFEs = 5000*Dimension;
   }
   if (!RunFlag)
   {
      EndRun = StartRun;
   }
   if (!FuncFlag)
   {
      EndFunction = StartFunction;
   }
   if (!SeedFlag)
   {
      RandSeed = time(NULL)+10*StartRun;
   }
   if (!IntervalFlag)
   {
      Interval = 500.0*sqrt(50)*60/Population/sqrt(Dimension);
   }

   if (optind < Argc)
   {
      fprintf(stderr, "Additional arguments will not be used\n");
   }
}

void cmdLnFinalize(void)
{
   /* nothing yet */
}

