
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include "src/f1tof6data.h"
#include "src/f1tof6func.h"


#define abss(a)    (((a) < 0)   ? (-(a)) : (a))
#define max(a, b)  (((a) > (b)) ?   (a)  : (b))
#define min(a, b)  (((a) > (b)) ?   (b)  : (a))

#define PI         (acos(-1.0))
#define EE         (exp(1.0))


long double Shifted_Sphere( int dim , double* x )
{
    int i;
    double z;
    long double F = 0;
    for(i=0;i<dim;i++)
    {
        z = x[i] - sphere[i];
        F += z*z;
    }
    
    return (F + f_bias[0]);
}

long double Schwefel_Problem(int dim, double *x)
{
    int i;
    double z;
    long double F = abss(x[0]);
    
    for(i = 1; i < dim; i++)
    {
    	  z = x[i] - schwefel[i];
        F = max(F, abss(z));
    }
    
    return (F + f_bias[1]);
}

long double Shifted_Rosenbrock( int dim , double* x )
{
    int i;
    double z[dim];
    long double F = 0;

    for(i=0;i<dim;i++) z[i] = x[i] - rosenbrock[i] + 1;   

    for(i=0;i<dim-1;i++)
    {    
        F = F + 100*( pow((pow(z[i],2)-z[i+1]) , 2) ) + pow((z[i]-1) , 2);
    }
    
    return (F + f_bias[2]);
}


long double Shifted_Rastrigin( int dim , double* x )
{
    int i;
    double z;
    long double F = 0;
    for(i=0;i<dim;i++){  
        z = x[i] - rastrigin[i];
        F = F + ( pow(z,2) - 10*cos(2*PI*z) + 10);
    }
    
    return (F + f_bias[3]);
}

long double Shifted_Griewank( int dim , double* x )
{
    int i;
    double z;
    long double F1 = 0;
    long double F2 = 1;
    for(i=0;i<dim;i++)
    {       
        z = x[i] - griewank[i];
        F1 = F1 + ( pow(z,2) / 4000 );
        F2 = F2 * ( cos(z/sqrt(i+1)));

    }
    
    return (F1 - F2 + 1 + f_bias[4]);
}

long double Shifted_Ackley( int dim , double* x )
{
    int i;
    double z;
    double Sum1 = 0;
    double Sum2 = 0;
    long double F = 0;
    for(i=0;i<dim;i++)
    {   
        z = x[i] - ackley[i];
        Sum1 = Sum1 + pow(z , 2 );
        Sum2 = Sum2 + cos(2*PI*z);
    }
    F = -20*exp(-0.2*sqrt(Sum1/dim)) - exp(Sum2/dim) + 20 + EE;

    return (F + f_bias[5]);
}

