
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "src/random.h"
#include "src/cmdln_gade.h"
#include "src/f1tof6data.h"
#include "src/f7tof19data.h"
#include "src/validate.h"
#include "src/gade_tools.h"
#include "src/benchmark_func.h"
#include "src/gade_operations.h"

static double mp[100000];
static double Fm1[100000];
static double Fm2[100000];
static double CRm1[100000];
static double CRm2[100000];
static long double Fitness[100000];
static long double eps = 1e-14;

static void dumpRunInfo(int FEs, int func, int run, int dim, int MaxFEs, GadePara *para)
{
   long double BestFitness;

   if (para->BestFitness < eps)
   {
      BestFitness = 0.0;
   }
   else
   {
      BestFitness = para->BestFitness;
   }

   printf("Func_num = %d, Run = %d, Dim = %d\n", func, run, dim);
   printf("FEs = %d (%.0lf%%)\n", FEs, FEs*100.0/MaxFEs);
   printf("BestFitness = %Lg\n", BestFitness);
   printf("mp = %lf\n", para->mp);
   printf("Fm[0] = %lf, Fm[1] = %lf\n", para->Fm[0], para->Fm[1]);
   printf("CRm[0] = %lf, CRm[1] = %lf\n", para->CRm[0], para->CRm[1]);
   printf("\n");
}


int main(int argc, char *argv[])
{
   int FEs;
   int run;
   int func;
   int Interval;
   struct GadePara para;
   struct GadeSetting gade;
   time_t tstart;
   time_t tend;
   int counter;

   cmdLnInit(argc, argv);

   Interval = cmdLnInterval();
   gade.Dimension = cmdLnDimension();
   gade.Population = cmdLnPopulation();
   gade.SAfactor = cmdLnSAfactor();
   gade.MaxFEs = cmdLnMaxFEs();
   gade.StartRun = cmdLnStartRun();
   gade.EndRun = cmdLnEndRun();
   gade.StartFunction = cmdLnStartFunction();
   gade.EndFunction = cmdLnEndFunction();

   randInit(cmdLnRandSeed());

   Individual  pop[gade.Population];
   Individual vpop[gade.Population];
   Individual upop[gade.Population];
   double F1[gade.Population];
   double F2[gade.Population];
   double CR1[gade.Population];
   double CR2[gade.Population];
   int flag1[gade.Population];
   int flag2[gade.Population];
   int index[gade.Population];
   long double DiffFitness[gade.Population];

   para.F[0] = F1; para.F[1] = F2;
   para.CR[0] = CR1; para.CR[1] = CR2;
   para.flag[0] = flag1; para.flag[1] = flag2;
   para.index = index;
   para.DiffFitness = DiffFitness;

   for (func = gade.StartFunction; func <= gade.EndFunction; func++)
   {
      for (run = gade.StartRun; run <= gade.EndRun; run++)
      {
         outputInit(func, run, gade.Dimension);
         outputRandSeed(getRandSeed());

         tstart = time(NULL);

         gadeInit(func, gade, &para, pop);
         
         benchmark_func(func, gade.Population, pop);
         FEs = gade.Population;

         para.BestFitness = gadeBestFitness(gade.Population, pop) - fbias[func];
         counter = 0;
         if (para.BestFitness < eps)
         {
            Fitness[counter] = 0.0;
         }
         else
         {
            Fitness[counter] = para.BestFitness;
         }
         mp[counter] = para.mp;
         Fm1[counter] = para.Fm[0];
         Fm2[counter] = para.Fm[1];
         CRm1[counter] = para.CRm[0];
         CRm2[counter] = para.CRm[1];

         while (FEs <= gade.MaxFEs-gade.Population)
         {
            if (Interval && (counter % Interval == 0))
            {
               dumpRunInfo(FEs, func, run, gade.Dimension, gade.MaxFEs, &para);
            }

            gadeSetParameters(&para);
            
            gadeMutation(gade.Population, para, pop, vpop);

            gadeCrossover(gade.Population, para, pop, vpop, upop);

            benchmark_func(func, gade.Population, upop);
            FEs = FEs + gade.Population;

            gadeSelection(gade.Population, pop, upop, &para);

            gadeAdaptParameters(&para);

            para.BestFitness = gadeBestFitness(gade.Population, pop) - fbias[func];
            counter++;
            if (para.BestFitness < eps)
            {
               Fitness[counter] = 0.0;
            }
            else
            {
               Fitness[counter] = para.BestFitness;
            }
            mp[counter] = para.mp;
            Fm1[counter] = para.Fm[0];
            Fm2[counter] = para.Fm[1];
            CRm1[counter] = para.CRm[0];
            CRm2[counter] = para.CRm[1];
         }

         counter++;
         dumpRunInfo(FEs, func, run, gade.Dimension, gade.MaxFEs, &para);

         gadeFinalize();

         tend = time(NULL);

         outputRunTime((tend-tstart)*1.0);
         outputFitnessValues(counter, Fitness);
         outputMPvalues(counter, mp);
         outputFMvalues(counter, Fm1, Fm2);
         outputCRMvalues(counter, CRm1, CRm2);
         outputFinalize();
      }
   }

   randFinalize();
   cmdLnFinalize();

   return 0;
}

