
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "src/random.h"
#include "src/benchmark_func.h"
#include "src/gade_operations.h"


static double L[] =
{
   0.0,        /* NULL */
   -100.0,     /* F1   */
   -100.0,     /* F2   */
   -100.0,     /* F3   */
   -5.0,       /* F4   */
   -600.0,     /* F5   */
   -32.0,      /* F6   */
   -10.0,      /* F7   */
   -65.536,    /* F8   */
   -100.0,     /* F9   */
   -15.0,      /* F10  */
   -100.0,     /* F11  */
   -100.0,     /* F12  */
   -100.0,     /* F13  */
   -5.0,       /* F14  */
   -10.0,      /* F15  */
   -100.0,     /* F16  */
   -100.0,     /* F17  */
   -5.0,       /* F18  */
   -10.0       /* F19  */
};

static double U[] =
{
   0.0,        /* NULL */
   100.0,      /* F1   */
   100.0,      /* F2   */
   100.0,      /* F3   */
   5.0,        /* F4   */
   600.0,      /* F5   */
   32.0,       /* F6   */
   10.0,       /* F7   */
   65.536,     /* F8   */
   100.0,      /* F9   */
   15.0,       /* F10  */
   100.0,      /* F11  */
   100.0,      /* F12  */
   100.0,      /* F13  */
   5.0,        /* F14  */
   10.0,       /* F15  */
   100.0,      /* F16  */
   100.0,      /* F17  */
   5.0,        /* F18  */
   10.0        /* F19  */
};


static void pbestIndex(int np, double p, Individual *pop, int *pr)
{
   int pnp;
   int id[np];
   int flag[np];
   int i, j, k;
   long double bestval;

   pnp = round(p*np);

   for (i = 0; i < np; i++)
   {
      flag[i] = 0;
   }
   for (i = 0; i < pnp; i++)
   {
      for (k = 0; flag[k] == 1; k++);
      id[i] = k;
      bestval = pop[k].val;
      
      for (j = k+1; j < np; j++)
      {
         if ((flag[j] == 0) && (pop[j].val < bestval))
         {
            bestval = pop[j].val;
            id[i] = j;
         }
      }
      flag[id[i]] = 1;
   }

   for (i = 0; i < np; i++)
   {
      pr[i] = id[discreteUniform(0, pnp-1)];
   }
}

static int compare(const void *A, const void *B)
{
   int elemA;
   int elemB;

   elemA = *((int *)A);
   elemB = *((int *)B);

   return (elemA >= elemB);
}

static void lpbestIndex(int np, double p, Individual *pop, int *pr)
{
   int i;
   int pnp;
   int id[np];
   int p1[np];
   int p2[np];

   randperm(np-1, p1+1, 1);
   randperm(np, p2, 0);
   pnp = round(p*np);

   p1[0] = 0;
   p1[pnp] = np;

   qsort(p1, pnp, sizeof(int), compare);

   for (i = 0; i < pnp; i++)
   {
      id[i] = gadeBestIndex(np, p1[i], p1[i+1]-1, p2, pop);
   }

   for (i = 0; i < np; i++)
   {
      pr[i] = id[discreteUniform(0, pnp-1)];
   }
}

void gadeInit(int func, GadeSetting gade, GadePara *para, Individual *pop)
{
   int i, j;
   
   para->p = 0.2;
   para->c = gade.SAfactor;
   para->mp = 0.5;
   para->np = gade.Population;
   para->Mc[0] = 0.0;
   para->Mc[1] = 0.0;
   para->Fm[0] = 0.5;
   para->Fm[1] = 0.5;
   para->CRm[0] = 0.9;
   para->CRm[1] = 0.9;

   for (i = 0; i < para->np; i++)
   {
      pop[i].dim = gade.Dimension;
      for (j = 0; j < pop[i].dim; j++)
      {
         pop[i].x[j] = uniform(L[func], U[func]);
      }
   }
}

void gadeSetParameters(GadePara *para)
{
   int i;

   for (i = 0; i < para->np; i++)
   {
      para->index[i] = 0;
      if (uniform(0,1) < para->mp)
      {
         para->flag[0][i] = 1;
         para->flag[1][i] = 0;
      }
      else
      {
         para->flag[0][i] = 0;
         para->flag[1][i] = 1;
      }
      para->F[0][i] = wrapcauchy(para->Fm[0], 0.2);
      para->F[1][i] = wrapcauchy(para->Fm[1], 0.2);
      para->CR[0][i] = 0.9;//wrapgaussian(para->CRm[0], 0.1);
      para->CR[0][i] = wrapgaussian(para->CRm[0], 0.1);
      para->CR[1][i] = 0.9;//wrapgaussian(para->CRm[1], 0.1);
      para->CR[1][i] = wrapgaussian(para->CRm[1], 0.1);
   }
}

void gadeMutation(int np, GadePara para, Individual *pop, Individual *vpop)
{
   int i, j;
   int ind[3];
   int r1[np];
   int r2[np];
   int r3[np];
   int r4[np];
   int pr[np];

   lpbestIndex(np, para.p, pop, pr);
   randperm(np, r1, 0);
   randperm(3, ind, 1);

   for (i = 0; i < np; i++)
   {
      r2[i] = (r1[i]+ind[0])%np;
      r3[i] = (r1[i]+ind[1])%np;
      r4[i] = (r1[i]+ind[2])%np;

      if (para.flag[0][i])
      {
         for (j = 0; j < pop[i].dim; j++)
         {
            vpop[i].x[j] = pop[r1[i]].x[j] + 
               (para.F[0][i]) * (pop[r2[i]].x[j] - pop[r3[i]].x[j]);
         }
      }
      else
      {
         for (j = 0; j < pop[i].dim; j++)
         {
            vpop[i].x[j] = pop[i].x[j] + 
               (para.F[1][i]) * (pop[pr[i]].x[j] - pop[i].x[j]) + 
               (para.F[1][i]) * (pop[r2[i]].x[j] - pop[r3[i]].x[j]) +
               (para.F[1][i]) * (pop[r1[i]].x[j] - pop[r4[i]].x[j]);
         }
      }
   }
}

void gadeCrossover(int np, GadePara para, Individual *pop, Individual *vpop, Individual *upop)
{
   double CR;
   int i, j, k;

   for (i = 0; i < np; i++)
   {
      if (para.flag[0][i])
      {
         CR = para.CR[0][i];
      }
      else
      {
         CR = para.CR[1][i];
      }
      
      upop[i] = pop[i];
     
      j = discreteUniform(0, pop[i].dim);
      upop[i].x[j] = vpop[i].x[j];

      for (k = 0; k < pop[i].dim-1; k++)
      {
         if (uniform(0, 1) > CR)
         {
            break;
         }
         j = (j+1)%(pop[i].dim);
         upop[i].x[j] = vpop[i].x[j];
      }
   }
}

void gadeSelection(int np, Individual *pop, Individual *upop, GadePara *para)
{
   int i; 

   for (i = 0; i < np; i++)
   {
      para->DiffFitness[i] = pop[i].val - upop[i].val;
      if (upop[i].val <= pop[i].val)
      {
         if (upop[i].val < pop[i].val)
         {
            para->index[i] = 1;
         }
         else
         {
            para->index[i] = 0;
         }
         pop[i] = upop[i];
      }
   }
}

void gadeAdaptParameters(GadePara *para)
{
   int i;
   double mp;
   double w[2];
   double Mg[2];
   double Fm[2];
   double CRm[2];
   double Rate[2];
   double ImpRate[2];
   long double SumDiffFitness[2];
   int sumflag[2];
   int index[2][para->np];

   
   SumDiffFitness[0] = 0.0;
   SumDiffFitness[1] = 0.0;
   for (i = 0; i < para->np; i++)
   {
      index[0][i] = (para->index[i])*(para->flag[0][i]);
      index[1][i] = (para->index[i])*(para->flag[1][i]);
      
      SumDiffFitness[0] = SumDiffFitness[0] + para->DiffFitness[i]*index[0][i];
      SumDiffFitness[1] = SumDiffFitness[1] + para->DiffFitness[i]*index[1][i];
   }

   Mg[0] = intsum(para->np, index[0]);
   if (Mg[0]+para->Mc[0] < 1.0)
   {
      w[0] = 0.0;
   }
   else
   {
      w[0] = Mg[0]/(Mg[0]+para->Mc[0]);
   }
   para->Mc[0] = (1-para->c)*(para->Mc[0]) + Mg[0];

   Mg[1] = intsum(para->np, index[1]);
   if (Mg[1]+para->Mc[1] < 1.0)
   {
      w[1] = 0.0;
   }
   else
   {
      w[1] = Mg[1]/(Mg[1]+para->Mc[1]);
   }
   para->Mc[1] = (1-para->c)*(para->Mc[1]) + Mg[1];

   Fm[0] = 0.0;
   CRm[0] = 0.0;
   if (SumDiffFitness[0] > 0.0)
   {
      for (i = 0; i < para->np; i++)
      {
         if (index[0][i])
         {
            Fm[0] = Fm[0] + (para->F[0][i])*(para->DiffFitness[i])/SumDiffFitness[0];
            CRm[0] = CRm[0] + (para->CR[0][i])*(para->DiffFitness[i])/SumDiffFitness[0];
         }
      }
      para->Fm[0] = (1-w[0])*(para->Fm[0]) + w[0]*Fm[0];
      para->CRm[0] = (1-w[0])*(para->CRm[0]) + w[0]*CRm[0];
   }
   
   Fm[1] = 0.0;
   CRm[1] = 0.0;
   if (SumDiffFitness[1] > 0.0)
   {
      for (i = 0; i < para->np; i++)
      {
         if (index[1][i])
         {
            Fm[1] = Fm[1] + (para->F[1][i])*(para->DiffFitness[i])/SumDiffFitness[1];
            CRm[1] = CRm[1] + (para->CR[1][i])*(para->DiffFitness[i])/SumDiffFitness[1];
         }
      }
      para->Fm[1] = (1-w[1])*(para->Fm[1]) + w[1]*Fm[1];
      para->CRm[1] = (1-w[1])*(para->CRm[1]) + w[1]*CRm[1];
   }

   sumflag[0] = intsum(para->np, para->flag[0]);
   sumflag[1] = intsum(para->np, para->flag[1]);
   if ((SumDiffFitness[0]+SumDiffFitness[1] > 0.0) && sumflag[0] && sumflag[1])
   {
      ImpRate[0] = SumDiffFitness[0]/sumflag[0];
      ImpRate[1] = SumDiffFitness[1]/sumflag[1];

      Rate[0] = ImpRate[0]/(ImpRate[0]+ImpRate[1]);
      Rate[1] = ImpRate[1]/(ImpRate[0]+ImpRate[1]);

      mp = Rate[0] / (Rate[0] + Rate[1]);
      para->mp = (1-para->c)*para->mp + (para->c)*mp;

      if (para->mp < 0.1)
      {
         para->mp = 0.1;
      }
      if (para->mp > 0.9)
      {
         para->mp = 0.9;
      }
   }
}

long double gadeBestFitness(int np, Individual *pop)
{
   int i;
   long double bestval;

   bestval = pop[0].val;
   for (i = 1; i < np; i++)
   {
      if (pop[i].val < bestval)
      {
         bestval = pop[i].val;
      }
   }

   return (bestval);
}

int gadeBestIndex(int np, int lower, int upper, int *r, Individual *pop)
{
   int i;
   int id;
   long double bestval;

   id = r[lower];
   bestval = pop[r[lower]].val;
   
   for (i = lower+1; i <= upper; i++)
   {
      if (pop[r[i]].val < bestval)
      {
         id = r[i];
         bestval = pop[r[i]].val;
      }
   }

   return (id);
}

void gadeFinalize(void)
{
   /* nothing yet */
}

