
#include <stdio.h>
#include <math.h>
#include "src/gade_tools.h"

static FILE *infoFp = NULL;
static FILE *resultFp = NULL;
static FILE *traceMPfp = NULL;
static FILE *traceFMfp = NULL;
static FILE *traceCRMfp = NULL;


int intsum(int n, int *flag)
{
   int i;
   int sum;

   sum = 0;
   for (i = 0; i < n; i++)
   {
      sum += flag[i];
   }

   return (sum);
}

double std(int n, double *x)
{
   int i;
   double r;
   double sum;

   sum = 0.0;
   r = mean(n, x);
   for (i = 0; i < n; i++)
   {
      sum = sum + (x[i]-r)*(x[i]-r);
   }

   return sqrt(sum/(n-1));
}

double mean(int n, double *x)
{
   int i;
   double sum;

   sum  = 0.0;
   for (i = 0; i < n; i++)
   {
      sum += x[i];
   }

   return (sum/n);
}

void outputInit(int func, int run, int dim)
{
   char FileName[160];

   snprintf(FileName, 160, "info/info_f%02dr%02dd%d.txt", func, run, dim);
   infoFp = fopen(FileName, "w");

   snprintf(FileName, 160, "result/fitness_f%02dr%02dd%d.txt", func, run, dim);
   resultFp = fopen(FileName, "w");
   
   snprintf(FileName, 160, "trace/mp_f%02dr%02dd%d.txt", func, run, dim);
   traceMPfp = fopen(FileName, "w");
   
   snprintf(FileName, 160, "trace/fm_f%02dr%02dd%d.txt", func, run, dim);
   traceFMfp = fopen(FileName, "w");
   
   snprintf(FileName, 160, "trace/crm_f%02dr%02dd%d.txt", func, run, dim);
   traceCRMfp = fopen(FileName, "w");
}

void outputRandSeed(unsigned int seed)
{
   if (infoFp)
   {
      fprintf(infoFp, "%u\n", seed);
   }
}

void outputRunTime(double second)
{
   if (infoFp)
   {
      fprintf(infoFp, "%.2lf\n", second);
   }
}

void outputMPvalues(int n, double *mp)
{
   int i;

   for (i = 0; i < n; i++)
   {
      if (traceMPfp)
      {
         fprintf(traceMPfp, "%lf\n", mp[i]);
      }
   }
}

void outputFMvalues(int n, double *Fm1, double *Fm2)
{
   int i;

   for (i = 0; i < n; i++)
   {
      if (traceFMfp)
      {
         fprintf(traceFMfp, "%lf %lf\n", Fm1[i], Fm2[i]);
      }
   }
}

void outputCRMvalues(int n, double *CRm1, double *CRm2)
{
   int i;

   for (i = 0; i < n; i++)
   {
      if (traceCRMfp)
      {
         fprintf(traceCRMfp, "%lf %lf\n", CRm1[i], CRm2[i]);
      }
   }
}

void outputFitnessValues(int n, long double *f)
{
   int i;

   for (i = 0; i < n; i++)
   {
      if (resultFp)
      {
         fprintf(resultFp, "%Lg\n", f[i]);
      }
   }
}

void outputFinalize(void)
{
   fclose(infoFp);
   fclose(resultFp);
   fclose(traceMPfp);
   fclose(traceFMfp);
   fclose(traceCRMfp);
}

