
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "src/random.h"

#define PI     (acos(-1.0))
static unsigned int RandSeed;


void randInit(int seed)
{
   if (seed > 0)
   {
      RandSeed = (unsigned int)seed;
   }
   else
   {
      RandSeed = time(NULL);
   }
   srand(RandSeed);
}

unsigned int getRandSeed(void)
{
   return (RandSeed);
}

double uniform(double lower, double upper)
{
   return (lower + rand()*(upper-lower)/RAND_MAX);
}

int discreteUniform(int lower, int upper)
{
   int temp;

   temp = floor(uniform(0,1) * (upper-lower+1));

   return (lower + (int)temp);
}

double gaussian(double mu, double sigma)
{
   static double v, fac;
   static int phase = 0;
   double S, Z, U1, U2, u;

   if (phase)
   {
      Z = v * fac;
   }
   else
   {
      do
      {
         U1 = rand()*1.0 / RAND_MAX;
         U2 = rand()*1.0 / RAND_MAX;

         u = 2.0 * U1 - 1.0;
         v = 2.0 * U2 - 1.0;
         S = u*u + v*v;
      } while (S >= 1.0);

      fac = sqrt(-2.0 * log(S) / S);
      Z = u * fac;
   }

   phase = 1 - phase;

   return (Z*sigma + mu);
}

double wrapgaussian(double mu, double sigma)
{
   double ran;

   do
   {
      ran = gaussian(mu, sigma);
   } while ((ran > 1.0) || (ran < 0.0));

   return ran;
}

double cauchy(double mu, double delta)
{
   return (mu + delta*tan(PI*(rand()*1.0/RAND_MAX - 0.5)));
}

double wrapcauchy(double mu, double delta)
{
   double ran;

   do
   {
      ran = cauchy(mu, delta);
   } while ((ran > 2.0) || (ran < 0.0));

   return (ran);
}

struct ElementPair
{
   int rid;
   double val;
};

static int compare(const void *A, const void *B)
{
   struct ElementPair *elemA;
   struct ElementPair *elemB;

   elemA = (struct ElementPair *)A;
   elemB = (struct ElementPair *)B;

   return (elemA->val <= elemB->val);
}

void randperm(int n, int *rid, int lower)
{
   int i;
   struct ElementPair elem[n];

   for (i = 0; i < n; i++)
   {
      elem[i].rid = lower + i;
      elem[i].val = uniform(0, 1);
   }

   qsort(elem, n, sizeof(struct ElementPair), compare);

   for (i = 0; i < n; i++)
   {
      rid[i] = elem[i].rid;
   }
}

void randFinalize(void)
{
   /* nothing yet */
}

